use std::fmt::Display;

use crate::fs::{AsIter, Mountpoint};

use super::{Stack, Stackable};

#[derive(Clone, Copy, Debug, PartialEq, Eq, PartialOrd, Ord)]
pub enum RedirectDir {
    On,
    Follow,
    NoFollow,
    Off,
}

#[derive(Clone, Copy, Debug, PartialEq, Eq, PartialOrd, Ord)]
pub enum Uuid {
    On,
    Auto,
    Null,
    Off,
}

#[derive(Clone, Copy, Debug, PartialEq, Eq, PartialOrd, Ord)]
pub enum Verity {
    On,
    Require,
    Off,
}

#[derive(Clone, Copy, Debug, PartialEq, Eq, PartialOrd, Ord)]
pub enum Xino {
    On,
    Auto,
    Off,
}

#[derive(Debug)]
pub enum OverlayOptions {
    Index(bool),
    Metacopy(bool),
    NfsExport(bool),
    RedirectDir(RedirectDir),
    UserXAttr,
    Uuid(Uuid),
    Verity(Verity),
    Volatile,
    Xino(Xino),
}

impl Display for OverlayOptions {
    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
        write!(
            f,
            "{}",
            match self {
                OverlayOptions::Index(enabled) => match enabled {
                    true => "index=on",
                    false => "index=off",
                },
                OverlayOptions::Metacopy(enabled) => match enabled {
                    true => "metacopy=on",
                    false => "metacopy=off",
                },
                OverlayOptions::NfsExport(enabled) => match enabled {
                    true => "nfs_export=on",
                    false => "nfs_export=off",
                },
                OverlayOptions::RedirectDir(redirect_dir) => match redirect_dir {
                    RedirectDir::On => "redirect_dir=on",
                    RedirectDir::Follow => "redirect_dir=follow",
                    RedirectDir::NoFollow => "redirect_dir=nofollow",
                    RedirectDir::Off => "redirect_dir=off",
                },
                OverlayOptions::UserXAttr => "userxattr",
                OverlayOptions::Uuid(uuid) => match uuid {
                    Uuid::On => "uuid=on",
                    Uuid::Auto => "uuid=auto",
                    Uuid::Null => "uuid=null",
                    Uuid::Off => "uuid=off",
                },
                OverlayOptions::Verity(verity) => match verity {
                    Verity::On => "verity=on",
                    Verity::Require => "verity=require",
                    Verity::Off => "verity=off",
                },
                OverlayOptions::Volatile => "volatile",
                OverlayOptions::Xino(xino) => match xino {
                    Xino::On => "xino=on",
                    Xino::Auto => "xino=auto",
                    Xino::Off => "xino=off",
                },
            }
        )
    }
}

#[derive(Debug)]
pub struct Overlay {
    fs: Stackable,
    options: Vec<OverlayOptions>,
}

impl Stack for Overlay {
    fn lower_dirs(&self) -> impl Iterator<Item = &std::path::Path> {
        self.fs.lower_dirs()
    }

    fn upper_dir(&self) -> Option<&std::path::Path> {
        self.fs.upper_dir()
    }

    fn work_dir(&self) -> Option<&std::path::Path> {
        self.fs.work_dir()
    }
}

impl Mountpoint for Overlay {
    fn options(&self) -> impl Iterator<Item = impl Display> {
        self.options.as_iter()
    }
}
